<?php
/*
	Plugin Name: Method
	Description: Плагин для оплаты через Method.
	Author: Paygine Developer
	Version: 2.0.0
	Edition: PAYMENT
 */

include_once 'Paygine.php';
new PayginePayment;

class PayginePayment
{
	private static $pluginName = 'payment-paygine'; // название плагина (соответствует названию папки)
	private static $pluginPublicName = 'Method'; // Публичное имя оплаты
	private $payment = [];
	public function __construct()
	{
		// Действие при активации плагина
		mgActivateThisPlugin(__FILE__, [__CLASS__, 'activate']);
		
		// Отрисовка формы оплаты (после оформления заказа)
		mgAddAction(self::$pluginName, [__CLASS__, 'getOrderForm'], 1);
		
		// Обработка webhooks (запросы, которые приходят с банков при совершении какого-то события)
		mgAddAction('Models_Payment_handleRequest', [__CLASS__, 'webhookIntercept'], 1);
	}
	
	// Этот метод срабатывает при активации плагина (При включении или активации оплаты)
	public static function activate()
	{
		$currentPayment = Models_Payment::getPaymentByPlugin(self::$pluginName);
		if(!$currentPayment){
			
			$icon = SITE . '/mg-plugins/' . self::$pluginName . '/src/icon.png';
			
			// Параметры оплаты
			$defaultParams = [
				[
					'name' => 'sector',
					'title' => 'Сектор',
					'type' => 'text',
					'value' => ''
				],
				[
					'name' => 'password',
					'title' => 'Пароль',
					'type' => 'text',
					'value' => ''
				],
				[
					'name' => 'test_mode',
					'title' => 'Тестовый режим',
					'type' => 'checkbox',
				],
				[
					'name' => 'payment_type',
					'title' => 'Способ оплаты',
					'type' => 'select',
					'value' => 'acquiring',
					'options' => [
						[
							'title' => 'Стандартный эквайринг (одностадийная оплата)',
							'value' => 'acquiring',
						],
						[
							'title' => 'Стандартный эквайринг (двухстадийная оплата)',
							'value' => 'acquiring_two_stage',
						],
						[
							'title' => 'Плайт (одностадийная оплата)',
							'value' => 'svkb',
						],
						[
							'title' => 'Плайт (двухстадийная оплата)',
							'value' => 'svkb_two_stage',
						],
						[
							'title' => 'Система Быстрых Платежей (СБП)',
							'value' => 'sbp',
						],
					],
				],
				[
					'name' => 'modal',
					'title' => 'Открывать пейформу в модальном окне',
					'type' => 'checkbox',
				],
				[
					'name' => 'tax',
					'title' => 'Налог',
					'type' => 'select',
					'value' => '6',
					'options' => [
						[
							'title' => 'ставка НДС 20%',
							'value' => '1',
						],
						[
							'title' => 'ставка НДС 10%',
							'value' => '2',
						],
						[
							'title' => 'ставка НДС расч. 20/120',
							'value' => '3',
						],
						[
							'title' => 'ставка НДС расч. 10/110',
							'value' => '4',
						],
						[
							'title' => 'ставка НДС 0%',
							'value' => '5',
						],
						[
							'title' => 'НДС не облагается',
							'value' => '6',
						]
					],
				],
				[
					'name' => 'hashing',
					'title' => 'Алгоритм шифрования данных',
					'type' => 'select',
					'value' => 'md5',
					'options' => [
						[
							'title' => 'MD5',
							'value' => 'md5',
						],
						[
							'title' => 'SHA256',
							'value' => 'sha256',
						],
					],
				],
				[
					'name' => 'custom_order_title',
					'title' => 'Настраиваемые статусы для заказов',
					'type' => 'checkbox',
				],
				[
					'name' => 'order_authorized',
					'title' => 'Средства заблокированы',
					'type' => 'select',
					'value' => Paygine::PAID,
					'options' => Paygine::getOrderStatusList(),
				],
				[
					'name' => 'order_completed',
					'title' => 'Платеж выполнен',
					'type' => 'select',
					'value' => Paygine::PAID,
					'options' => Paygine::getOrderStatusList(),
				],
				[
					'name' => 'order_canceled',
					'title' => 'Платеж отменен',
					'type' => 'select',
					'value' => Paygine::CANCELED,
					'options' => Paygine::getOrderStatusList(),
				],
			];
			
			// info - голубая, success - зелёная, fail - красная, warning - желтая
			$urls = [
				[
					'type' => 'info',
					'title' => 'Notyfy URL',
					'link' => '/ajaxrequest?mguniqueurl=action/notification&pluginHandler=' . self::$pluginName,
				],
			];
			
			Models_Payment::addPayment(
				self::$pluginName,
				self::$pluginPublicName,
				self::$pluginPublicName,
				self::$pluginName,
				$defaultParams,
				$icon,
				0,
				$urls
			);
		}
	}
	
	public static function getOrderForm($args)
	{
		$result = $args['result'];
		
		// Получаем данные о заказе
		$order_id = $args['args'][1];
		$orderModel = new Models_Order();
		$orders = $orderModel->getOrder('`id` = ' . DB::quoteInt($order_id));
		$order = $orders[$order_id];
		// При работе плагинов скидок или снижения цен,
		// стоимость товаров или стоимость заказа может иметь число с более двух цифр после запятой,
		// чтобы устранить этот эффект, содержимое заказа рекомендуется получать через метод getCorrectOrderContent,
		// а сумму заказа округлять через round($order['summ'], 2)
		// перед передачей параметров заказа в банк
		$orderContent = $orderModel->getCorrectOrderContent($order);
		$payment = Models_Payment::getPaymentByPlugin(self::$pluginName);
		$payment_param = $payment['paramArray'];
		
		$pc_order_id = Paygine::registerOrder($payment_param, $order, $orderContent);
		
		if(intval($pc_order_id) == 0) {
			Models_Payment::loger(self::$pluginName, __CLASS__ . '_' . __FUNCTION__, ['error' => $pc_order_id]);
			$xml = @simplexml_load_string($pc_order_id);
			return $xml ? json_encode($xml, JSON_UNESCAPED_UNICODE) : $pc_order_id;
		}
		
		
		if(method_exists($orderModel, 'updateOrder')) {
			$orderModel->updateOrder(['id' => $order_id, 'status_id' => Paygine::EXPECTS_PAYMENT]);
		}
		
		$server_host = Paygine::getPaymentUrl($payment_param['test_mode']);
		$payment_path = Paygine::getPaymentPath($payment_param['payment_type']);
		$payment_url = $server_host . $payment_path;
		$args = [
			'sector' => $payment_param['sector'],
			'id' => $pc_order_id,
		];
		
		if(strpos($payment_path, 'skvb') !== false) {
			$shop_cart = Paygine::calcShopCart($order, $orderContent);
			if($shop_cart)
				$args['shop_cart'] = base64_encode(json_encode($shop_cart, JSON_UNESCAPED_UNICODE));
		}
		
		Paygine::signData($args, $payment_param['hashing'], $payment_param['password']);
		$modal_url = $payment_url . "?" . http_build_query($args);
		// Генерируем вёрстку из views/form.php
		ob_start();
		require('views/form.php');
		$result = ob_get_contents();
		ob_end_clean();
		
		return $result;
	}
	
	// Обработка запросов на контроллер payment. (Как правило это webhooks от банков)
	public static function webhookIntercept($args)
	{
		$result = $args['result'];
		// Проверяем что запрос адресован этому плагину
		if($_GET['payment'] !== self::$pluginName) return $result;
		// Получаем оплату плагина
		$payment = Models_Payment::getPaymentByPlugin(self::$pluginName);
		$payment_param = $payment['paramArray'];
		
		// js redirect from iframe to main window
		if(!empty($payment_param['modal']) && empty($_REQUEST['skip_redirect']))
			Paygine::jsRedirect(SITE . $_SERVER['REQUEST_URI'] . '&skip_redirect=1');
		
		try{
			if(!empty($_REQUEST['error']))
				throw new Exception('В процессе оплаты заказа возникла ошибка (error ' . $_REQUEST['error']. ')');
			
			$pc_order_id = !empty($_REQUEST['id']) ? (int)$_REQUEST['id'] : 0;
			$pc_operation_id = !empty($_REQUEST['operation']) ? (int)$_REQUEST['operation'] : 0;
			$order_id = !empty($_REQUEST['reference']) ? (int)$_REQUEST['reference'] : 0;
			
			if(!$pc_order_id || !$pc_operation_id || !$order_id){
				throw new Exception('Отсутствуют данные для обработки платежа');
			}
			
			$order = Paygine::getOrder($order_id, $payment['id']);
			if(!$order)
				throw new Exception('Не удалось найти заказ');
			
			//if((int)$order['status_id'] !== self::EXPECTS_PAYMENT)
			//	throw new Exception('Статус заказа не соответствует статусу "Ожидает оплаты"');
			
			$data = [
				'sector' => $payment_param['sector'],
				'id' => $pc_order_id,
			];
			Paygine::signData($data, $payment_param['hashing'], $payment_param['password']);
			$response = Paygine::sendRequest(Paygine::getPaymentUrl($payment_param['test_mode']) . '/webapi/Order', $data);
			$pc_order = Paygine::parseXML($response);
			Paygine::responseValidation($pc_order, $payment_param, false);
			
			if((int)$pc_order['id'] !== $pc_order_id)
				throw new Exception('Номер заказа не совпадает с номером заказа в ПЦ');
			
			if((int) $pc_order['amount'] !== (Paygine::centify($order['summ']) + Paygine::centify($order['delivery_cost'])))
				throw new Exception('Сумма заказа не совпадает с суммой заказа в ПЦ');
			
		} catch(Exception $e) {
			// По возможности логируем взаимодействие плагина с банковской системой
			Models_Payment::loger(self::$pluginName, __CLASS__ . '_' . __FUNCTION__, $_REQUEST);
			return [
				'status' => 'fail',
				'message' => '<div class="c-alert c-alert--red">' . $e->getMessage() . ')</div>',
			];
		}
		
		if($pc_order['state'] === Paygine::ORDER_COMPLETED || $pc_order['state'] === Paygine::ORDER_AUTHORIZED){
			Controllers_Payment::actionWhenPayment([
				'paymentOrderId' => $order_id,
				'paymentAmount' => $order['summ'] + $order['delivery_cost'],
				'paymentID' => $payment['id']
			]);
			$result = [
				'status' => 'success',
				'message' => 'Вы успешно оплатили заказ. Спасибо!',
			];
		} else {
			$result = [
				'status' => 'fail',
				'message' => 'При попытке оплаты заказа произошла ошибка.<br />Пожалуйста, попробуйте позже или используйте другой способ оплаты.<br />'.
				'<br /><a class="c-button" href="' . SITE . '/order?p=' . $order['pay_hash'] . '">Страница заказа</a>',
			];
		}
		
		return $result;
	}
	
}

?>